﻿Imports Autodesk.AutoCAD.Runtime
Imports Autodesk.AutoCAD.ApplicationServices
Imports Autodesk.AutoCAD.DatabaseServices
Imports Autodesk.AutoCAD.Geometry
Imports Autodesk.AutoCAD.EditorInput

Public Class Gia_Bach
    <CommandMethod("AddHatch")> Public Sub AddHatch()
        '' Get the current document and database
        Dim acDoc As Document = Application.DocumentManager.MdiActiveDocument
        Dim acCurDb As Database = acDoc.Database

        '' Start a transaction
        Using acTrans As Transaction = acCurDb.TransactionManager.StartTransaction()

            '' Open the Block table for read
            Dim acBlkTbl As BlockTable
            acBlkTbl = acTrans.GetObject(acCurDb.BlockTableId, OpenMode.ForRead)

            '' Open the Block table record Model space for write
            Dim acBlkTblRec As BlockTableRecord
            acBlkTblRec = acTrans.GetObject(acBlkTbl(BlockTableRecord.ModelSpace), OpenMode.ForWrite)

            '' Create a circle object for the closed boundary to hatch
            Dim acCirc As Circle = New Circle()
            acCirc.SetDatabaseDefaults()
            acCirc.Center = New Point3d(3, 3, 0)
            acCirc.Radius = 1

            '' Add the new circle object to the block table record and the transaction
            acBlkTblRec.AppendEntity(acCirc)
            acTrans.AddNewlyCreatedDBObject(acCirc, True)

            '' Adds the circle to an object id array
            Dim acObjIdColl As ObjectIdCollection = New ObjectIdCollection()
            acObjIdColl.Add(acCirc.ObjectId)

            '' Create the hatch object and append it to the block table record
            Dim acHatch As Hatch = New Hatch()
            acBlkTblRec.AppendEntity(acHatch)
            acTrans.AddNewlyCreatedDBObject(acHatch, True)

            '' Set the properties of the hatch object
            '' Associative must be set after the hatch object is appended to the 
            '' block table record and before AppendLoop
            acHatch.SetDatabaseDefaults()
            acHatch.SetHatchPattern(HatchPatternType.PreDefined, "ANSI31")
            acHatch.Associative = True
            acHatch.AppendLoop(HatchLoopTypes.Outermost, acObjIdColl)
            acHatch.EvaluateHatch(True)

            '' Save the new object to the database
            acTrans.Commit()
        End Using
    End Sub

    <CommandMethod("test")> Public Sub MakeHatch()
        Dim adoc As Document = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument
        Dim db As Database = adoc.Database
        Dim doclock As DocumentLock = adoc.LockDocument
        adoc.LockDocument(DocumentLockMode.Write, Nothing, Nothing, True)
        Using tr As Transaction = db.TransactionManager.StartTransaction
            Dim ed As Editor = Autodesk.AutoCAD.ApplicationServices.Application.DocumentManager.MdiActiveDocument.Editor
            Try

                Dim preOp As PromptEntityOptions = New PromptEntityOptions(vbCrLf & "Select Curve to be hatch: ")
                preOp.AllowNone = False
                preOp.SetRejectMessage(vbCrLf & "Need to select a curve!")
                preOp.AddAllowedClass(GetType(Curve), False)

                Dim res As PromptEntityResult = ed.GetEntity(preOp)
                If res.Status <> PromptStatus.OK Then
                    Return
                End If
                Dim ent As Entity = tr.GetObject(res.ObjectId, OpenMode.ForRead, False)
                Dim ids As ObjectIdCollection = New ObjectIdCollection
                ids.Add(res.ObjectId)
                Dim patName As String = "ANSI32"
                Dim btr As BlockTableRecord = CType(tr.GetObject(db.CurrentSpaceId, OpenMode.ForWrite), BlockTableRecord)
                Dim hatchObj As New Hatch
                btr.AppendEntity(hatchObj)
                tr.AddNewlyCreatedDBObject(hatchObj, True)
                hatchObj.SetHatchPattern(HatchPatternType.PreDefined, patName)
                hatchObj.Associative = True
                hatchObj.AppendLoop(HatchLoopTypes.Outermost, ids)
                tr.Commit()
            Catch ex As Autodesk.AutoCAD.Runtime.Exception
                ed.WriteMessage(ex.StackTrace)
            Finally
                tr.Dispose()
            End Try
        End Using
        doclock.Dispose()
    End Sub

End Class